#!/bin/sh

SOC_FAMILY="mx6ul"

MTD_ROOT=/dev/mtd0
MTD_UBOOT_SPL=/dev/mtd1
MTD_UBOOT=/dev/mtd2
MTD_UBOOT_ENV=/dev/mtd3
MTD_UBOOT_ENV_REDUNDANT=""

EMMC_DEV=/dev/mmcblk0
EMMC_PART=/dev/mmcblk0p1

UBOOT_BIN=u-boot.bin
UBOOT_ENV=uboot-env.txt
SPL_BIN=flash.bin

ROOTFS_IMG="" 
BOOTFS_IMG="" 
USERFS_IMG=""

ROOT_UBI=ubifs.img
ROOT_EXT=rootfs.ext3

IMAGE_WIC=image.wic.gz

prod_initramfs=initramfs
prod_linux=zImage.bin
prod_linux_dtb=zImage.dtb

# UBI with two volumes 0 and 1
# Name of file with ubi0_0 volume contents
UBI_VOL0_NAME=
# Size of first volume in MiB (second volume occupies the rest of ubi)
UBI_VOL0_SIZE_MIB=

FW_DIR="/tmp/fw_imx"
FW_LABEL="boot"
MNT_DIR="/mnt"
USB_DEV="/dev/sda1"

export LANG=en_US.UTF8

if [ -z "$SRC_MEDIA" ]; then
	SRC_MEDIA="FAT_BOOT_PART"
fi

#check images with md5sum (md5sum.IMAGE_NAME.txt  file must be available)
images_md5=""
MD5_UBOOT_BIN=$MD5_PRE$UBOOT_BIN
MD5_UBOOT_ENV=$MD5_PRE$UBOOT_ENV
MD5_ROOT_UBI=$MD5_PRE$ROOT_UBI
MD5_ROOT_EXT=$MD5_PRE$ROOT_EXT

DEV_UBI=/dev/ubi0
maxEraseCnt=1

PROD_LOG=/tmp/prodLog.txt

#default TFTP Server configuration
TFTP_SERVER=192.168.1.10
TFTP_PROD_DIR=$TFTP_FW_DIR/Production
TFTP_BLOCKSIZE=65464

#default NFS Server configuration
NFS_SERVER=192.168.1.10

FSL_OTP=/sys/fsl_otp

# for A/B Root-filesystem (swupdate)
label_rootfs_a="root_A"
label_rootfs_b="root_B"
label_bootfs="boot"
label_userfs="data"
rootfs_size=150 
bootfs_size=30
userfs_size=""
swupdate_bin=/usr/bin/swupdate
update_A="stable,root_A"
update_B="stable,root_B"
UPDATE_CFG="/etc/swupdate.cfg"
SWU_ARCHIVE="swupdate-img.swu" 

if [ -e /usr/share/production/prod-env.sh ]; then
	source /usr/share/production/prod-env.sh
else
	echo " environment settings missing (/usr/share/production/prod-env.sh), using defaults!"
fi

MD5_PRE="$FW_DIR/md5sum."

if [ "$SOC_FAMILY" = "mx6ul" ]; then
	########################### OTP ########################
	########### OTP register map
	# 0x00 LOCK
	# 0x01 CFG0
	# 0x02 CFG1
	# 0x03 CFG2
	# 0x04 CFG3
	# 0x05 CFG4 (Boot config)
	# 0x06 CFG5 (Boot config)
	# ...
	# 0x22 MAC0
	# 0x23 MAC1
	# 0x24 MAC2 (UL only)
	# ...
	# 0x26 GP1
	# 0x27 GP2

	# accessing otp register via fsl_opt and sysfs
	SYSFS_LOCK_OTP=$FSL_OTP/HW_OCOTP_LOCK
	SYSFS_BOOT_CONFIG_OTP=$FSL_OTP/HW_OCOTP_CFG4
	SYSFS_BT_FUSE_SEL_OTP=$FSL_OTP/HW_OCOTP_CFG5
	SYSFS_MAC0_OTP=$FSL_OTP/HW_OCOTP_MAC0
	SYSFS_MAC1_OTP=$FSL_OTP/HW_OCOTP_MAC1
	SYSFS_MAC2_OTP=$FSL_OTP/HW_OCOTP_MAC2
	SYSFS_GP1_OTP=$FSL_OTP/HW_OCOTP_GP1
	SYSFS_GP2_OTP=$FSL_OTP/HW_OCOTP_GP2

	# accessing otp register via imx-ocotp
	# /sys/bus/nvmem/devices/imx-ocotp0/nvmem
	LOCK_OCOTP_REG=0
	BOOT_CONFIG_OCOTP_REG=5
	BOOT_CONFIG2_OCOTP_REG=6
	MAC0_OCOTP_REG=34
	MAC1_OCOTP_REG=35
	MAC2_OCOTP_REG=36
	GP1_OCOTP_REG=38
	GP2_OCOTP_REG=39

	BOOT_CONFIG2=0x10
	MAC_LOCK=0x100
	GP1_LOCK=0x400
	GP2_LOCK=0x1000
elif [ "$SOC_FAMILY" = "mx8mm" ]; then
	LOCK_OCOTP_REG=0
	BOOT_CONFIG_OCOTP_REG=7
	BOOT_CONFIG2_OCOTP_REG=8
	MAC0_OCOTP_REG=36
	MAC1_OCOTP_REG=37
	MAC2_OCOTP_REG=38
	GP1_OCOTP_REG=56
	GP2_OCOTP_REG=58

	MAC_LOCK=0x4000
	GP1_LOCK=0x100000
	GP2_LOCK=0x400000
fi

########################################################################
# set content of OTP Register no $1 to hex value $2
setOtpRegister() {
	if [ "$#" != 2 ]; then
		printStatus "$# parameters set: <$@>"
		printStatus "need 2 parameters: register number 0-127 and hex value" 1
		printStatus "example for setting bootconfig: setOtpRegister 5 0x0x49002040"
		return 1
	fi
	# check number
	case $1 in
	*[!0-9]* | '')
		printStatus "Parameter 1 not a number" 1
		return 1
		;;
	esac
	if [ $1 -gt 127 ]; then
		printStatus "Parameter $1 not in range" 1
		return 1
	fi
	#checking hex value
	nPrefix=$(echo $2 | cut -c -2)
	if [ $nPrefix != "0x" ]; then
		printStatus "need 0x as prefix, not $nPrefix" 1
		return 1
	fi
	# check hex number
	case $2 in
	0x*[!0-9a-fA-F]* | '')
		printStatus "Error: Parameter 2 not a hex value: <$2>" 1
		return 1
		;;
	esac
	szHexNumber=$(printf '%08x' $2)

	# accessing OTP via fsl-otp driver ...
	if [ -d $FSL_OTP ]; then
		# accessing OTP via fsl-otp driver ...
		case $1 in
		$LOCK_OCOTP_REG)
			OTP_REG=$SYSFS_LOCK_OTP
			;;
		$BOOT_CONFIG_OCOTP_REG)
			OTP_REG=$SYSFS_BOOT_CONFIG_OTP
			;;
		$BOOT_CONFIG2_OCOTP_REG)
			OTP_REG=$SYSFS_BT_FUSE_SEL_OTP
			;;
		$MAC0_OCOTP_REG)
			OTP_REG=$SYSFS_MAC0_OTP
			;;
		$MAC1_OCOTP_REG)
			OTP_REG=$SYSFS_MAC1_OTP
			;;
		$MAC2_OCOTP_REG)
			OTP_REG=$SYSFS_MAC2_OTP
			;;
		$GP1_OCOTP_REG)
			OTP_REG=$SYSFS_GP1_OTP
			;;
		$GP2_OCOTP_REG)
			OTP_REG=$SYSFS_GP2_OTP
			;;
		*)
			printStatus "Error: Register $1 unknown"
			return 1
			;;
		esac

		if [ ! -e $OTP_REG ]; then
			printStatus "$OTP_REG not available" 1
			return 1
		else
			echo $szHexNumber >$OTP_REG
			#           echo "schreibe $szHexNumber nach $OTP_REG"
		fi
	else
		# accessing OTP via imx-ocotp mainline driver in nvmem
		echo "OTP Register: <$1>"
		echo "Value=<$szHexNumber>"
		szHex="\\x"
		szHexFormat=$szHex$(echo $szHexNumber | cut -c 7-8)
		szHexFormat=$szHexFormat$szHex$(echo $szHexNumber | cut -c 5-6)
		szHexFormat=$szHexFormat$szHex$(echo $szHexNumber | cut -c 3-4)
		szHexFormat=$szHexFormat$szHex$(echo $szHexNumber | cut -c -2)
		#echo "szHexFormat=<$szHexFormat>"
		printf "%b" $szHexFormat | dd of=/sys/bus/nvmem/devices/imx-ocotp0/nvmem seek=$1 count=1 bs=4 conv=notrunc 2>/dev/null
	fi
}

########################################################################
# get content of OTP Register no $1
getOtpRegister() {

	if [ -d $FSL_OTP ]; then
		# accessing OTP via fsl-otp driver ...
		case $1 in
		$LOCK_OCOTP_REG)
			OTP_REG=$SYSFS_LOCK_OTP
			;;
		$BOOT_CONFIG_OCOTP_REG)
			OTP_REG=$SYSFS_BOOT_CONFIG_OTP
			;;
		$BOOT_CONFIG2_OCOTP_REG)
			OTP_REG=$SYSFS_BT_FUSE_SEL_OTP
			;;
		$MAC0_OCOTP_REG)
			OTP_REG=$SYSFS_MAC0_OTP
			;;
		$MAC1_OCOTP_REG)
			OTP_REG=$SYSFS_MAC1_OTP
			;;
		$MAC2_OCOTP_REG)
			OTP_REG=$SYSFS_MAC2_OTP
			;;
		$GP1_OCOTP_REG)
			OTP_REG=$SYSFS_GP1_OTP
			;;
		$GP2_OCOTP_REG)
			OTP_REG=$SYSFS_GP2_OTP
			;;
		*)
			printStatus "Error: Register $1 unknown"
			return 1
			;;
		esac

		if [ ! -e $OTP_REG ]; then
			printStatus "$OTP_REG not available" 1
			return 1
		else
			printf '0x%08x\n' $(cat $OTP_REG)
		fi

	else
		# accessing OTP via imx-ocotp mainline driver in nvmem
		#	hexdump -v -e '"0x%08x\n"' /sys/bus/nvmem/devices/imx-ocotp0/nvmem | sed -n "$reg_off"p
		dd if=/sys/bus/nvmem/devices/imx-ocotp0/nvmem skip=$1 count=1 bs=4 2>/dev/null | hexdump -e '"0x%08x\n"'

	fi
}

########################################################################
#Fehlernummern:
# 0: Flashen erfolgreich
# 1: Image zum Flashen fehlt
# 2: Checksumme fehlt
# 3: Checksummen-Fehler
flashResult() {
	component=$1
	status=$2
	if [ "$status" != 0 ]; then
		echo "Error: flashing $1 failed (ErrorNo: $2)" | tee -a $PROD_LOG
	else
		echo "OK: flashing $1 success" | tee -a $PROD_LOG
	fi
}

printStatus() {
	component=$1
	status=$2
	if [ $# -eq 1 ]; then
		echo "Log: $1" | tee -a $PROD_LOG
	elif [ "$status" != 0 ]; then
		echo "Error: $1" | tee -a $PROD_LOG
	else
		echo "OK: $1" | tee -a $PROD_LOG
	fi
}

########################################################################
run_prod() {
	if [ -x "/usr/share/production/production-prod.sh" ]; then
		/bin/sh /usr/share/production/production-prod.sh $@
	else
		printStatus "Production script not found (maybe you need to install production-tool-prod)!" 1
		exit 1
	fi
}

########################################################################
check_md5() {
	STATUS=0
	if [ "$images_md5" == "yes"  ];then	
		if [ ! -s $MD5_PRE$1 ]; then
			printStatus "$MD5_PRE$1 is missing" 1
			STATUS=2
			return $STATUS
		fi
		if [ "$(md5sum $FW_DIR/$1 | cut -d ' ' -f 1)" != "$(cat $MD5_PRE$1 | cut -d ' ' -f 1)" ]; then
			printStatus "md5 checksum error of $1" 1
			STATUS=3
			return $STATUS
		fi
	else 
		printStatus "omitting md5sum check due to config setting (/usr/share/production/prod-env.sh)"
		return 0
	fi
}

########################################################################
run_test() {
	if [ $(type -t $1) == "$1" ]; then
		$@
	elif [ $(type -t $1) == "function" ]; then
		$@
	else
		echo "Error: Test not available"
	fi
}
########################################################################
enable_dynamic_eth() {
	if [ "$3" == "1" ]; then
		ifconfig $1 down 1>>$PROD_LOG 2>&1

		mac=$2
		l=${#mac}

		if [ ! $l -eq 12 ]; then
			printStatus "Invalid MAC length!" 1
			return 1
		fi

		# set MAC temporarily now to avoid reboot
		ifconfig $1 hw ether ${mac:0:2}:${mac:2:2}:${mac:4:2}:${mac:6:2}:${mac:8:2}:${mac:10:2} 1>>$PROD_LOG 2>&1
	fi

	ifconfig $1 up 1>>$PROD_LOG 2>&1

	# fetch IP from DHCP server
	printStatus "Ordering IP address from dhcp server ..."
	ifconfig $1 dynamic 1>>$PROD_LOG 2>&1
	udhcpc -i $1 -n 1>>$PROD_LOG 2>&1
	STATUS=$?
	printStatus "connection with dhcp server " $STATUS
	return $STATUS
}

cleanup_update_testlinux() {
	umount $MNT_DIR/nfs
	umount $MNT_DIR/usb
}

########################################################################
# Description update_testlinux:
# update usb stick from nfs if testlinux differs
update_testlinux() {
	STATUS=0
	if [ ! $(cat /proc/cmdline | grep "fw_load_state=out_of_date" | wc -c) -gt 0 ]; then
		printStatus "fw_load_state=out_of_date"
		# creating directories
		if [ ! -d $MNT_DIR ]; then
			mkdir $MNT_DIR
		fi
		if [ ! -d $MNT_DIR/nfs ]; then
			mkdir $MNT_DIR/nfs
		fi
		if [ ! -d $MNT_DIR/usb ]; then
			mkdir $MNT_DIR/usb
		fi
		# mounting nfs and usb stick
		if [ -n "$NFS_SERVER" ] && [ -n "$NFS_TESTLINUX_DIR" ]; then
			if ! mountpoint -q "$MNT_DIR/nfs"; then
				printStatus "update_testlinux: Mounting NFS ..."
				mount -t nfs $NFS_SERVER:$NFS_TESTLINUX_DIR $MNT_DIR/nfs 1>>$PROD_LOG 2>&1
				STATUS=$?
				if [ ! $STATUS -eq 0 ]; then
					printStatus "update_testlinux: mount nfs $NFS_SERVER:$NFS_TESTLINUX_DIR to $MNT_DIR/nfs failed " 1
					return 1
				fi
			fi
			if ! mountpoint -q $MNT_DIR/usb; then
				printStatus "update_testlinux: Mounting usb stick ..."
				mount $USB_DEV $MNT_DIR/usb 1>>$PROD_LOG 2>&1
				STATUS=$?
				if [ ! $STATUS -eq 0 ]; then
					umount $MNT_DIR/nfs
					printStatus "update_testlinux: mount usb stick $USB_DEV to $MNT_DIR/usb failed " 1
					return 1
				fi
			fi
		else
			printStatus "update_testlinux: NFS-server environment not set!" 1
			return 1
		fi
		# compare md5 sums and if different copy file(s) from NFS Server to usb stick
		for file in $prod_initramfs $prod_linux $prod_linux_dtb; do
			let do_copy=0
			if [ -s $MNT_DIR/nfs/$file.md5 -a -s $MNT_DIR/nfs/$file ]; then
				if [ ! -s $MNT_DIR/usb/$file.md5 ]; then
					printStatus "file $MNT_DIR/usb/$file.md5 missing"
					let do_copy=1
				else
					printStatus "comparing md5 files of $file"
					cmp -s $MNT_DIR/nfs/$file.md5 $MNT_DIR/usb/$file.md5
					if [ $? -ne 0 ]; then
						printStatus "md5 differs ($file) "
						let do_copy=1
					else
						printStatus "file $file up to date"
					fi
				fi
			else
				printStatus "file $MNT_DIR/nfs/$file.md5 or $MNT_DIR/nfs/$file not found"
				cleanup_update_testlinux
				return 1
			fi
			if [ $do_copy -gt 0 ]; then
				cp $MNT_DIR/nfs/$file.md5 $MNT_DIR/usb/$file.md5
				STATUS=$?
				cp $MNT_DIR/nfs/$file $MNT_DIR/usb/$file
				let STATUS=STATUS+$?
				sync
				let STATUS=STATUS+$?
				if [ ! $STATUS -eq 0 ]; then
					printStatus "copy file $file failed"
					cleanup_update_testlinux
					return 1
				else
					printStatus "file $file copied"
				fi
			fi
		done

	else
		printStatus "update_testlinux: no fw_load_state found"
	fi
	cleanup_update_testlinux
	printStatus "update_testlinx" $STATUS
	return $STATUS
}

########################################################################
# Description getProductionFiles:
# depending on global variable SRC_MEDIA this functions does following:
# SRC_MEDIA == FAT_BOOT_PART:
#	- mount partition named "boot" on /mnt and create syslink /tmp/production -> /mnt

# SRC_MEDIA == SAMBA_DRIVE:
# 	- create directory and copy file(s) (must be defined as parameters) from samba drive to /tmp/production
#	- to successfully copy files from netdevice further global variables must be set:
#		- SAMBA_SERVER:  Address of Server e.g. export SAMBA_SERVER=//172.17.10.21/pmb_hdd
#		- SAMBA_USER: e.g. export SAMBA_USER=pmb
#		- SAMBA_PWD: e.g.export SAMBA_PWD=pmb_exceet
# 		- SAMBA_DIR: the directory to copy the file from e.g. export SAMBA_DIR=Linux_Produktions_Daten/test

getProductionFiles() {
	if [ "$SRC_MEDIA" == "FAT_BOOT_PART" ]; then
		if ! mountpoint -q $MNT_DIR; then
			printStatus "Mounting FAT ..."
			mount /dev/disk/by-label/$FW_LABEL $MNT_DIR 1>>$PROD_LOG 2>&1
			ln -s $MNT_DIR $FW_DIR
	    	else                                                                                                                            
		    if [ ! -h $FW_DIR ]; then                               
		    	ln -s $MNT_DIR $FW_DIR                          
		    fi                                                      
        	fi 
	else
		if [ "$#" -eq 0 ]; then
			printStatus "pleace specify file to download when $SRC_MEDIA is used" 1
			return 1
		fi
		# create production directory if it does not exist
		if [ ! -d $FW_DIR ]; then
			mkdir $FW_DIR
			if [ $? -ne 0 ]; then
				printStatus "could not create destination directory $FW_DIR" 1
				return 1
			fi
		fi
		if [ "$SRC_MEDIA" == "SAMBA_DRIVE" ]; then
			# copy file(s) from samba netserver
			for file in "$@"; do
				if [ -n "$SAMBA_SERVER" ] || [ -n "$SAMBA_DIR" ] || [ -n "$SAMBA_USER" ] || [ -n "$SAMBA_PWD" ]; then
					printStatus "copy production file $file from netdevice to $FW_DIR"
					smbclient $SAMBA_SERVER -U $SAMBA_USER%$SAMBA_PWD -D $SAMBA_DIR -c "lcd $FW_DIR; get $file" 1>>$PROD_LOG 2>&1
					STATUS=$?
					printStatus "copy production file $file" $STATUS
				else
					printStatus "SAMBA-server environment not set!" 1
					return 1
				fi
			done
		elif [ "$SRC_MEDIA" == "TFTP_DRIVE" ]; then
			# copy file(s) from TFTP Server
			for file in "$@"; do
				if [ -n "$TFTP_SERVER" ] && [ -n "$TFTP_FW_DIR" ]; then
					tftp -b $TFTP_BLOCKSIZE -g -r $TFTP_PROD_DIR/$file -l $FW_DIR/$file $TFTP_SERVER
					STATUS=$?
					printStatus "copy file from $TFTP_PROD_DIR/$file to $FW_DIR/$file"
				else
					printStatus "TFTP-server environment not set!" 1
					return 1
				fi
			done
		elif [ "$SRC_MEDIA" == "NFS_DRIVE" ]; then
			if [ -n "$NFS_SERVER" ] && [ -n "$NFS_FW_DIR" ]; then
				if ! mountpoint -q $MNT_DIR; then
					printStatus "Mounting NFS ..."
					mount -t nfs $NFS_SERVER:$NFS_FW_DIR $MNT_DIR 1>>$PROD_LOG 2>&1
					STATUS=$?
				fi
			else
				printStatus "NFS-server environment not set!" 1
				return 1
			fi
			# copy file(s) from NFS Server
			for file in "$@"; do
				cp $MNT_DIR/Production/$file $FW_DIR
				STATUS=$?
				printStatus "copy file from $MNT_DIR/production/$file to $FW_DIR"
			done
			umount $MNT_DIR
		elif [ "$SRC_MEDIA" == "HTTP_SERVER" ]; then
			if [ -n "$HTTP_URL" ]; then
				# copy file(s) from HTTP Server
				for file in "$@"; do
					if [ ! -f "$FW_DIR/$file" ]; then
						wget -q "$HTTP_URL/$file" -O "$FW_DIR/$file"
						STATUS=$?
						printStatus "download file from $HTTP_URL/$file to $FW_DIR/$file"
					fi
				done
			else
				printStatus "HTTP-server environment not set!" 1
				return 1
			fi
		fi
		return $STATUS
	fi
	return 0
}

clean_fw_dir() {
	if [ ! "$SRC_MEDIA" == "FAT_BOOT_PART" ] && [ ! "$SRC_MEDIA" == "HTTP_SERVER" ]; then
		rm -r $FW_DIR
	fi
}

########################################################################
uploadLog() {
	STATUS=0

	if [ "$SRC_MEDIA" == "SAMBA_DRIVE" ]; then
		if [ -z "$SAMBA_SERVER" ] || [ -z "$SAMBA_LOG_DIR" ] || [ -z "$SAMBA_USER" ] || [ -z "$SAMBA_PWD" ] || [ -z "$LOG_FILENAME" ]; then
			printStatus "environment for SAMBA-Server/Log not set!" 1
			return 1
		else
			printStatus "copy production log to network share ..."
			smbclient $SAMBA_SERVER -c "lcd $MNT_DIR; prompt OFF; recurse ON; cd $SAMBA_LOG_DIR; put $PROD_LOG $LOG_FILENAME" -U $SAMBA_USER%$SAMBA_PWD 1>>$PROD_LOG 2>&1
			STATUS=$?
			printStatus "copy production log" $STATUS
			printStatus "copy system log to network share ..."
			smbclient $SAMBA_SERVER -c "lcd $MNT_DIR; prompt OFF; recurse ON; cd $SAMBA_LOG_DIR; put /var/log/messages syslog-$LOG_FILENAME" -U $SAMBA_USER%$SAMBA_PWD 1>>$PROD_LOG 2>&1
			STATUS=$?
			printStatus "copy system log" $STATUS
			return $STATUS
		fi
	elif [ "$SRC_MEDIA" == "TFTP_DRIVE" ]; then
		if [ -z "$TFTP_SERVER" ] || [ -z "$TFTP_LOG_DIR" ] || [ -z "$LOG_FILENAME" ]; then
			printStatus "TFTP-server environment/Log not set!" 1
			return 1
		else
			printStatus "copy production/test log to tftp drive ..."
			tftp -b $TFTP_BLOCKSIZE -p -l $PROD_LOG -r $TFTP_LOG_DIR/$LOG_FILENAME $TFTP_SERVER
			STATUS=$?
			printStatus "copy production/test log to tftp drive" $STATUS
			return $STATUS
		fi
	fi
}

##############################################################################
#	check if device is mounted and umount it
#	input: device, either with absolute or relative path name to /dev
# 
#	assumption: none
#	return: 0 on succes 
#			1 on fail
##############################################################################
umount_dev(){
	local device=$1
	local devicenode

	if test ! -z "$(mount | grep ${device})";then
		printStatus "Unmount ${device}"
		for devicenode in /dev/${device} ${device} failed; do
			umount ${devicenode} 1>>$PROD_LOG 2>&1
			if test $? = 0; then
				# umount was succesfully
				break
			fi
			if test "${devicenode}" == "failed"; then
				printStatus "Unmounting device ${device} failed!" 1
				return 1
			fi
		done
	fi

	return 0
}

##############################################################################
#	check if ubi device mounted and umount it
#	input: ubi device
# 
#	assumption: none
#	return: 0 on succes 
#			1 on fail
##############################################################################
umount_dev_ubi_partitions(){
	local device=$1
	
	for i in 0 1 2 3; do
		umount_dev ${device}_$i
		if test $? != 0; then
			printStatus "Umount partition ${device}_$i failed" 1
			return 1
		fi
	done

	return 0
}

umount_dev_mmc_partitions(){
	local device=$1
	
	for i in 0 1 2 3; do
		umount_dev ${device}p$i
		if test $? != 0; then
			printStatus "Umount partition ${device}p$i failed" 1
			return 1
		fi
	done

	return 0
}

##############################################################################
#	** Initial Partitioning ** 
# prepare NAND Storage with Partitions needed for seamless AB update
# -> directory /home/root/fw must be equipped with Images (userfs/rootfs/bootfs)
##############################################################################
flash_ubi_AB() {
	local status=0
	local cur_par=$(findmnt -n -o SOURCE /)
	local calc_size_userfs=1
	local temp_var1=1
	local leb_size=1
	local leb_count=1
	local total_size=0

	if [ -z "$ROOTFS_IMG" ]; then
	    printStatus "rootfs image variable (ROOTFS_IMG) not set. " 1
	    return 1
	fi
	
	if [ -z "$USERFS_IMG" ]; then
	    printStatus "userfs image variable (USERFS_IMG)  not set. " 1
	    return 1
	fi
	
	if [ -z "$BOOTFS_IMG" ]; then
	    printStatus "bootfs image variable (BOOTFS_IMG) not set. " 1
	    return 1
	fi

	getProductionFiles $ROOTFS_IMG $BOOTFS_IMG $USERFS_IMG
	
	if [ ! -f $FW_DIR/$ROOTFS_IMG ]; then
		printStatus "$ROOTFS_IMG  not available in $FW_DIR" 1
		return 1
	fi
	
	if [ ! -f $FW_DIR/$BOOTFS_IMG ]; then
		printStatus "$BOOTFS_IMG  not available in $FW_DIR" 1
		return 1
	fi
	
	if [ ! -f $FW_DIR/$USERFS_IMG ]; then
		printStatus "$USERFS_IMG  not available in $FW_DIR" 1
		return 1
	fi
	
	
	if [ "${cur_par}" == "ubi0:root_A" -o "${cur_par}" == "ubi0:root_B" ];then
		printStatus "current root-fs is $cur_par. Boot from other media (mmc, network) to continue." 1 
		return 1
	fi
	
	
	#check if UBI Volumes are attached 
	if [ ! -z "$(ls /dev/ | grep ubi0)" ];then
		printStatus "Ubi device $MTD_ROOT is already attached, detaching it."
		umount_dev_ubi_partitions ubi0
		ubidetach -p $MTD_ROOT 1>/dev/null 2>&1
			if test $? != 0; then
				printStatus "Detaching device <${MTD_ROOT}> failed!" 1
				return 1
			fi
		fi
		
	#flash ubi volume
	ubiformat  -q -y $MTD_ROOT
	if test $? != 0; then
		printStatus "Ubiformat failed for device $MTD_ROOT!" 1
		return 1
	fi

	#attach ubi 
	ubiattach -p $MTD_ROOT 2>/dev/null
	if test $? != 0; then
		printStatus "Ubiattach failed for device $MTD_ROOT!" 1
		return 1
	fi
	
	# calculate size of ubi volume
	# subtract 2*rootfs_size and bootfs_size to get the available free space 
	# for the userfs Partition 
	
	leb_size=$(ubinfo /dev/ubi0 | awk 'NR==3 {print $4}')
	leb_count=$(ubinfo /dev/ubi0 | awk 'NR==5 {print $6}')

	total_size=$(echo $((leb_size * $leb_count)))
	total_size_mb=$(expr $total_size / 1024 / 1024 )
	if [ -z "$userfs_size" ]; then
	    userfs_size=$(expr $total_size / 1024 / 1024 - $(($rootfs_size * 2)) - $bootfs_size)
	fi
	printStatus "calculated total_size: $total_size_mb MiB"
	printStatus "calculated userfs_size: $userfs_size MiB"
	
	
	# abort when userfs_size is smaller than 10Mb

	if [ $userfs_size -lt 10 ]; then
		printStatus "userfs_size is smaller than 10Mb" 1
		printStatus "please reduce rootfs_size to leave room for userfs partition" 1 
		return 1
	fi
	
	ubimkvol /dev/ubi0 -s "$bootfs_size"MiB -N $label_bootfs
	let status=status+$?
	ubimkvol /dev/ubi0 -s "$rootfs_size"MiB -N $label_rootfs_a
	let status=status+$?
	ubimkvol /dev/ubi0 -s "$rootfs_size"MiB -N $label_rootfs_b
	let status=status+$?
	ubimkvol /dev/ubi0 -s "$userfs_size"MiB -N $label_userfs
	let status=status+$?
	
	if [ $status -ne 0 ]; then
		printStatus "failed to create ubi Volumes" 1 
		return $status
	fi
	


	# flash images on UBI Volumes
	
	ubi_update_volume $FW_DIR/$BOOTFS_IMG /dev/ubi0_0
	let status=status+$?
	ubi_update_volume $FW_DIR/$ROOTFS_IMG /dev/ubi0_1
	let status=status+$?
	ubi_update_volume $FW_DIR/$USERFS_IMG /dev/ubi0_3
	let status=status+$?
	
	if [ $status -ne 0 ]; then
	    printStatus "failed to update UBI volumes" 1 
	fi
		
	flashResult UBI-AB $status
	return $status
	

	return $status
	}



##############################################################################
#	** Initial Partitioning ** 
#	prepare NAND Storage with Partitions needed for using A/B 
#	Update approach
# 	
	#parameter: please provide path to a valid SWUpdate archive
#	* the .swu archive should provide rootfs.tar.gz and fitImage a file 
#	* build the image by using bitbake swupate-img
#	
##############################################################################

flash_ubi_swu() {
	local status=0
	local cur_par=$(findmnt -n -o SOURCE /)
	local calc_size_userfs=1
	local temp_var1=1
	local leb_size=1
	local leb_count=1
	local total_size=0

	getProductionFiles $SWU_ARCHIVE
	
	if [ ! -f $FW_DIR/$SWU_ARCHIVE ]; then
		printStatus "$SWU_ARCHIVE not available in $FW_DIR" 1
		return 1
		
	fi 
	
	
	if [ "${cur_par}" == "ubi0:root_A" -o "${cur_par}" == "ubi0:root_B" ];then
		printStatus "current root-fs is $cur_par. Boot from other media (mmc, network) to continue." 1 
		return 1
	fi
	
	
	#check if UBI Volumes are attached 
	if [ ! -z "$(ls /dev/ | grep ubi0)" ];then
		printStatus "Ubi device $MTD_ROOT is already attached, detaching it."
		umount_dev_ubi_partitions ubi0
		ubidetach -p $MTD_ROOT 1>/dev/null 2>&1
			if test $? != 0; then
				printStatus "Detaching device <${MTD_ROOT}> failed!" 1
				return 1
			fi
		fi
		
	#flash ubi volume
	ubiformat  -q -y $MTD_ROOT
	if test $? != 0; then
		printStatus "Ubiformat failed for device $MTD_ROOT!" 1
		return 1
	fi

	#attach ubi 
	ubiattach -p $MTD_ROOT 2>/dev/null
	if test $? != 0; then
		printStatus "Ubiattach failed for device $MTD_ROOT!" 1
		return 1
	fi
	
	# calculate size of ubi volume
	# subtract 2*rootfs_size and bootfs_size to get the available free space 
	# for the userfs Partition 
	
	leb_size=$(ubinfo /dev/ubi0 | awk 'NR==3 {print $4}')
	leb_count=$(ubinfo /dev/ubi0 | awk 'NR==5 {print $6}')

	total_size=$(echo $((leb_size * $leb_count)))
	total_size_mb=$(expr $total_size / 1024 / 1024 )
	if [ -z "$userfs_size" ]; then
	    userfs_size=$(expr $total_size / 1024 / 1024 - $(($rootfs_size * 2)) - $bootfs_size)
	fi
	printStatus "calculated total_size: $total_size_mb MiB"
	printStatus "calculated userfs_size: $userfs_size MiB"
	
	
	# abort when userfs_size is smaller than 10Mb

	if [ $userfs_size -lt 10 ]; then
		printStatus "userfs_size is smaller than 10Mb" 1
		printStatus "please reduce rootfs_size to leave room for userfs partition" 1 
		return 1
	fi
	
	ubimkvol /dev/ubi0 -s "$bootfs_size"MiB -N $label_bootfs
	let status=status+$?
	ubimkvol /dev/ubi0 -s "$rootfs_size"MiB -N $label_rootfs_a
	let status=status+$?
	ubimkvol /dev/ubi0 -s "$rootfs_size"MiB -N $label_rootfs_b
	let status=status+$?
	ubimkvol /dev/ubi0 -s "$userfs_size"MiB -N $label_userfs
	let status=status+$?
	
	if [ $status -ne 0 ]; then
		printStatus "failed to create ubi Volumes" 1 
		return $status
	fi
	
	if [ ! -x $swupdate_bin ];then 
		printStatus "$swupdate_bin ist not installed or executable." 1
		return 1
	fi
	
	# only possible to install root_A if we do initial partitioning/setup!!!
	printStatus "installing $SWU_ARCHIVE. Select $update_A"
	$swupdate_bin -f $UPDATE_CFG -e $update_A -i $FW_DIR/$SWU_ARCHIVE
	let status=status+$?
	#disable bootcounter in u-boot - no fallback installation in production setup. 
	fw_setenv upgrade_available 0
	fw_setenv bootcount 0
	flashResult UBI-SWU $status
	return $status
	}


########################################################################
flash_ubi() {
	local status=0
	local cur_par=$(mount | grep root |awk '{print $1}')
	local calc_size_userfs=1
	local temp_var1=1
	local leb_size=1
	local leb_count=1
	local total_size=0

	getProductionFiles $SWU_ARCHIVE
	
	if [ ! -f $FW_DIR/$SWU_ARCHIVE ]; then
		printStatus "$SWU_ARCHIVE not available in $FW_DIR" 1
		return 1
		
	fi 
	
	
	if [ "${cur_par}" == "ubi0:root_A" -o "${cur_par}" == "ubi0:root_B" ];then
		printStatus "current root-fs is $cur_par. Boot from other media (mmc, network) to continue." 1 
		return 1
	fi
	
	
	#check if UBI Volumes are attached 
	if [ ! -z "$(ls /dev/ | grep ubi0)" ];then
		printStatus "Ubi device $MTD_ROOT is already attached, detaching it."
		umount_dev_ubi_partitions ubi0
		ubidetach -p $MTD_ROOT 1>/dev/null 2>&1
			if test $? != 0; then
				printStatus "Detaching device <${MTD_ROOT}> failed!" 1
				return 1
			fi
		fi
		
	#flash ubi volume
	ubiformat  -q -y $MTD_ROOT
	if test $? != 0; then
		printStatus "Ubiformat failed for device $MTD_ROOT!" 1
		return 1
	fi

	#attach ubi 
	ubiattach -p $MTD_ROOT 2>/dev/null
	if test $? != 0; then
		printStatus "Ubiattach failed for device $MTD_ROOT!" 1
		return 1
	fi
	
	
	leb_size=$(ubinfo /dev/ubi0 | awk 'NR==3 {print $4}')
	leb_count=$(ubinfo /dev/ubi0 | awk 'NR==5 {print $6}')

	total_size=$(echo $((leb_size * $leb_count)))
	total_size_mb=$(expr $total_size / 1024 / 1024 )
	rootfs_size=$(($total_size_mb - $bootfs_size))
	
	printStatus "calculated total_size: $total_size_mb MiB"
	printStatus "calculated rootfs_size: $rootfs_size MiB"

	ubimkvol /dev/ubi0 -s "$bootfs_size"MiB -N $label_bootfs
	let status=status+$?
	ubimkvol /dev/ubi0 -s "$rootfs_size"MiB -N $label_rootfs_a
	let status=status+$?
	
	if [ $status -ne 0 ]; then
		printStatus "failed to create ubi Volumes" 1 
		return $status
	fi
	
	if [ ! -x $swupdate_bin ];then 
		printStatus "$swupdate_bin ist not installed or executable." 1
		return 1
	fi
	
	# only possible to install root_A if we do initial partitioning/setup!!!
	printStatus "installing $SWU_ARCHIVE. Select $update_A"
	$swupdate_bin -f $UPDATE_CFG -e $update_A -i $FW_DIR/$SWU_ARCHIVE
	let status=status+$?
	#disable bootcounter in u-boot - no fallback installation in production setup. 
	fw_setenv upgrade_available 0
	fw_setenv bootcount 0
	flashResult UBI-SWU $status
	return $status
}

########################################################################
flash_ubi_volumes() {
	getProductionFiles ${UBI_VOL0_IMAGE} md5sum.${UBI_VOL0_IMAGE}

	STATUS=0
	printStatus "writing ubi volumes ..."

	if [ ! -f $FW_DIR/${UBI_VOL0_IMAGE} ]; then
		printStatus "File ${UBI_VOL0_IMAGE} not found!" 1
		STATUS=1
		return $STATUS
	fi

	printStatus "checking md5 ..."
	check_md5 ${UBI_VOL0_IMAGE}
	STATUS=$?
	if [ "$STATUS" != 0 ]; then
		printStatus "md5 of file ${UBI_VOL0_IMAGE} does not match"
		return $STATUS
	fi

	printStatus "formatting ${MTD_ROOT} for ubi  ..."
	ubiformat $MTD_ROOT -y 1>>$PROD_LOG 2>&1
	if [ "$?" != 0 ]; then
		STATUS=1
		printStatus "ubiformat failed"
		return $STATUS
	fi

	printStatus "attaching ubi to ${MTD_ROOT} ..."
	ubiattach -p $MTD_ROOT 1>>$PROD_LOG 2>&1
	if [ "$?" != 0 ]; then
		STATUS=1
		printStatus "ubiattach failed"
		return $STATUS
	fi

	# create ubi volumes
	printStatus "creating ubi volume 'rootfs' with size ${UBI_VOL0_SIZE_MIB}MiB ..."
	ubimkvol /dev/ubi0 -N rootfs -s ${UBI_VOL0_SIZE_MIB}MiB
	printStatus "creating empty ubi volume 'data' ..."
	ubimkvol /dev/ubi0 -N data -m
	# fill in data
	printStatus "wirting contents for ubi volume from ${UBI_VOL0_IMAGE} file"
	ubiupdatevol /dev/ubi0_0 $FW_DIR/${UBI_VOL0_IMAGE}

	printStatus "detaching ubi..."
	ubidetach -p $MTD_ROOT 1>>$PROD_LOG 2>&1

	flashResult UBI-ROOTFS $STATUS
	clean_fw_dir
	return $STATUS
}

########################################################################
flash_bootloader() {
	getProductionFiles $UBOOT_BIN md5sum.$UBOOT_BIN
	STATUS=0
	
	printStatus "flashing bootloader ..."
	if [ -f $FW_DIR/$UBOOT_BIN ]; then
		check_md5 $UBOOT_BIN
		STATUS=$?
		if [ "$STATUS" = 0 ]; then
			flash_erase $MTD_UBOOT 0 0 1>>$PROD_LOG 2>&1
			STATUS=$?
			if [ $STATUS -eq 0 ]; then
				flashcp $FW_DIR/$UBOOT_BIN $MTD_UBOOT 1>>$PROD_LOG 2>&1
				STATUS=$?
			fi
		fi
	else
		printStatus "$UBOOT_BIN not found!" 1
		STATUS=1
	fi
	flashResult U-Boot $STATUS
	clean_fw_dir
	return $STATUS
}

########################################################################
flash_bootloader_spl() {
	getProductionFiles $SPL_BIN md5sum.$SPL_BIN
	STATUS=0
	printStatus "flashing SPL..."
	if [ -f $FW_DIR/$SPL_BIN ]; then
		check_md5 $SPL_BIN
		STATUS=$?
		if [ "$STATUS" = 0 ]; then
			# Bootloader Bereich im NOR-Flash löschen
			flash_erase $MTD_UBOOT_SPL 0 0 1>>$PROD_LOG 2>&1
			STATUS=$?
			if [ $STATUS -eq 0 ]; then
				dd if=$FW_DIR/$SPL_BIN of=$MTD_UBOOT_SPL seek=2 bs=512 1>>$PROD_LOG 2>&1
				STATUS=$?
			fi
		fi
	else
		printStatus "$SPL_BIN not found!" 1
		STATUS=1
	fi
	flashResult U-Boot-SPL $STATUS
	clean_fw_dir
	return $STATUS
}

########################################################################
flash_bootloadervars() {
	getProductionFiles $UBOOT_ENV md5sum.$UBOOT_ENV
	STATUS=0
	printStatus "flashing bootloader vars ..."
	if [ -f $FW_DIR/$UBOOT_ENV ]; then
		check_md5 $UBOOT_ENV
		STATUS=$?
		if [ "$STATUS" = 0 ]; then
			# Block 8 (64kB) mit U-Boot Vars loeschen
			printStatus "Erasing Uboot-Env ..."
			flash_erase $MTD_UBOOT_ENV 0 0 1>>$PROD_LOG 2>&1
			if [ "$MTD_UBOOT_ENV_REDUNDANT" ];then 
				printStatus "Erasing redundant Uboot-Env ..."
		 		flash_erase $MTD_UBOOT_ENV_REDUNDANT 0 0 1>>$PROD_LOG 2>&1
		 		# Uboot vars mit 64k erstellen (multiple copies)
		 		mkenvimage -r -s 0x10000 -o /tmp/ubootvars.bin $FW_DIR/$UBOOT_ENV 1>>$PROD_LOG 2>&1
		 	else 
		 		# Uboot vars mit 64k erstellen
				mkenvimage -s 0x10000 -o /tmp/ubootvars.bin $FW_DIR/$UBOOT_ENV 1>>$PROD_LOG 2>&1
			fi

			printStatus "Flashing Environment ..."
			flashcp /tmp/ubootvars.bin $MTD_UBOOT_ENV 1>>$PROD_LOG 2>&1
			STATUS=$?
			if [ "$MTD_UBOOT_ENV_REDUNDANT" ];then 
				printStatus "Flashing redundant Environment ..."
				flashcp /tmp/ubootvars.bin $MTD_UBOOT_ENV_REDUNDANT 1>>$PROD_LOG 2>&1
				let STATUS=STATUS+$?
			fi
			
		fi
	else
		printStatus "$UBOOT_ENV not found!" 1
		STATUS=1
	fi
	flashResult Bootloader-Variablen $STATUS
	clean_fw_dir
	return $STATUS
}


##############################################################################
# helper function: make mkfs.ext4 on given partition
##############################################################################
format_to_ext4() {

	local update_vol=$1
	local status=0

	if [ 0 != $(mount | grep $update_vol | wc -c) ]; then
		printStatus "unmounting $update_vol"
	    umount $update_vol
	fi	

	mkfs.ext4 -F $update_vol  1>>$PROD_LOG 2>&1
	status=$?
	return $status
}

##############################################################################
#	ubi update volume
#	input: img volume 
# 
#	assumption: img exists and volume exists ubi attached
#	return: 0 on succes 
#			1 on fail
##############################################################################
ubi_update_volume(){
	
	local image=$1
	local update_vol=$2
	local status=0
	
	echo "updating vol $update_vol"
	mkdir -p /tmp/ubi_tmp 1>>$PROD_LOG 2>&1
	mkfs.ubifs -F -y $update_vol  1>>$PROD_LOG 2>&1
	mount -t ubifs $update_vol /tmp/ubi_tmp  1>>$PROD_LOG 2>&1
	tar xf $image -C /tmp/ubi_tmp/ 
	status=$?
	umount $update_vol 2>/dev/null 1>>$PROD_LOG 2>&1
	
	rm -rf /tmp/ubi_tmp 1>>$PROD_LOG 2>&1
	
	
	return $status
}


##############################################################################
#	ext4 update volume
#	input: img volume 
# 
#	assumption: img exists and volume exists 
#	return: 0 on succes 
#			1 on fail
##############################################################################
ext4_update_volume(){
	
	local image=$2
	local update_vol=$1
	local status=0
	
	mkdir -p /tmp/ext4_tmp
	mkfs.ext4 -I 256 -F $update_vol  1>>$PROD_LOG 2>&1
	mount -t ext4 $update_vol /tmp/ext4_tmp 1>>$PROD_LOG 2>&1
	tar xf $image -C /tmp/ext4_tmp  
	status=$?	
	umount $update_vol 1>>$PROD_LOG 2>&1
	rm -rf /tmp/ext4_tmp 1>>$PROD_LOG 2>&1
	
	printStatus "updating vol $update_vol"
	return $status
}
##############################################################################


##############################################################################
#	** Initial Partitioning ** 
# prepare eMMC Storage with Partitions needed for seamless AB update
# -> directory /home/root/fw must be equipped with Images (userfs/rootfs/bootfs)
#	
##############################################################################
flash_emmc_AB() {
	local status=0
	local cur_par=$(findmnt -n -o SOURCE /)
	local total_size=0
	
	if [ "${cur_par}" == "${EMMC_DEV}p2" -o "${cur_par}" == "${EMMC_DEV}p3" ];then
		printStatus "current root-fs is $cur_par. Boot from other media (mmc, network) to continue." 1 
		return 1
	fi
	
	
	if [ -z "$ROOTFS_IMG" ]; then
	    printStatus "rootfs image variable (ROOTFS_IMG) not set. " 1
	    return 1
	fi
	
	if [ -z "$USERFS_IMG" ]; then
	    printStatus "userfs image variable (USERFS_IMG)  not set. " 1
	    return 1
	fi
	
	if [ -z "$BOOTFS_IMG" ]; then
	    printStatus "bootfs image variable (BOOTFS_IMG) not set. " 1
	    return 1
	fi

	getProductionFiles $ROOTFS_IMG $BOOTFS_IMG $USERFS_IMG
	
	if [ ! -f $FW_DIR/$ROOTFS_IMG ]; then
		printStatus "$ROOTFS_IMG  not available in $FW_DIR" 1
		return 1
	fi
	
	if [ ! -f $FW_DIR/$BOOTFS_IMG ]; then
		printStatus "$BOOTFS_IMG  not available in $FW_DIR" 1
		return 1
	fi
	
	if [ ! -f $FW_DIR/$USERFS_IMG ]; then
		printStatus "$USERFS_IMG  not available in $FW_DIR" 1
		return 1
	fi

	umount_dev_mmc_partitions `basename $EMMC_DEV`

	# calculate size of partitions on eMMC
	# subtract 2*rootfs_size and bootfs_size to get the available free space 
	# for the userfs Partition 

	total_size=$(sfdisk -l ${EMMC_DEV}  | grep "Disk /dev/"  | cut -d ' ' -f5)
	total_size_mb=$(expr $total_size / 1024 / 1024 )

	echo "totalsize: $total_size"
	echo "totalsize: $total_size_mb M"
	
	if [ -z "$userfs_size" ]; then
	    userfs_size=$(expr $total_size / 1024 / 1024 - $(($rootfs_size * 2)) - $bootfs_size)
	fi
	
	printStatus "calculated total_size: $total_size_mb MiB"
	printStatus "calculated userfs_size: $userfs_size MiB"

	dd if=/dev/zero of=${EMMC_DEV} bs=512 count=1 1>/dev/null 2>&1
	printStatus "creating partition table"
	sfdisk ${EMMC_DEV} 1>>$PROD_LOG 2>&1  << EOF
label:dos
size=${bootfs_size}M,
size=${rootfs_size}M,
size=${rootfs_size}M,
size=${userfs_size}M,
EOF

	for i in 1 2 3 4 ; do
		format_to_ext4 ${EMMC_DEV}p${i}
		let status=status+$?
	done

	if [ $status -ne 0 ]; then
		printStatus "failed to format ext4 partitions" 1 
		return $status
	fi

	# flash images on eMMC partitions
	
	ext4_update_volume ${EMMC_DEV}p1 $FW_DIR/$BOOTFS_IMG 
	let status=status+$?
	ext4_update_volume ${EMMC_DEV}p2 $FW_DIR/$ROOTFS_IMG 
	let status=status+$?
	ext4_update_volume ${EMMC_DEV}p4 $FW_DIR/$USERFS_IMG 
	let status=status+$?
	
	if [ $status -ne 0 ]; then
	    printStatus "failed to update ext4 volumes" 1 
	fi
		
	flashResult EMMC-AB $status
	return $status
}

########################################################################
flash_emmc() {
	STATUS=0
	getProductionFiles $ROOT_EXT.gz md5sum.$ROOT_EXT
	# unpack ext4.gz if available (only in production tftp mode
	if [ "$SRC_MEDIA" == "TFTP_DRIVE" ]; then
		gunzip $FW_DIR/$ROOT_EXT.gz
		STATUS=$?
	fi
	printStatus "flashing eMMC..."
	if [ -f $FW_DIR/$ROOT_EXT ]; then
		check_md5 $ROOT_EXT
		STATUS=$?
		if [ $STATUS -eq 0 ]; then
			if [ 0 != $(mount | grep $EMMC_PART | wc -c) ]; then
				printStatus "unmounting $EMMC_PART"
				umount $EMMC_PART
			fi

			dd if=/dev/zero of=${EMMC_DEV} bs=512 count=1 1>/dev/null 2>&1
			printStatus "creating partition table"
			sfdisk ${EMMC_DEV} 1>>$PROD_LOG 2>&1 <<EOF
label:dos
;
EOF
			STATUS=$?
			if [ $STATUS -eq 0 ]; then
				printStatus "dd $FW_DIR/$ROOT_EXT to $EMMC_PART"
				dd if=$FW_DIR/$ROOT_EXT of=$EMMC_PART bs=1M 1>>$PROD_LOG 2>&1
				let STATUS=STATUS+$?
				sync
			fi
			if [ $STATUS -eq 0 ]; then
				printStatus "resizing partition ..."
				e2fsck -f $EMMC_PART 1>>$PROD_LOG 2>&1
				resize2fs -p $EMMC_PART 1>>$PROD_LOG 2>&1
				let STATUS=STATUS+$?
				if [ $STATUS -ne 0 ]; then
					printStatus "rootfs on $EMMC_PART could not be extended to it's maximum size!" 1
				fi
			fi
		fi
	else
		printStatus "No ext3-Image $ROOT_EXT found in $FW_DIR" 1
		STATUS=1
	fi
	flashResult eMMC-ROOTFS $STATUS
	clean_fw_dir
	return $STATUS
}

########################################################################
flash_emmc_wic() {
	STATUS=0
	getProductionFiles $IMAGE_WIC md5sum.$IMAGE_WIC
	printStatus "flashing eMMC..."
	if [ -f $FW_DIR/$IMAGE_WIC ]; then
		check_md5 $IMAGE_WIC
		STATUS=$?
		if [ $STATUS -eq 0 ]; then
			if [ 0 != $(mount | grep $EMMC_PART | wc -c) ]; then
				printStatus "unmounting $EMMC_PART"
				umount $EMMC_PART
			fi

			printStatus "dd $FW_DIR/$IMAGE_WIC to $EMMC_DEV"
			gunzip -c $FW_DIR/$IMAGE_WIC | dd of=$EMMC_DEV bs=1M iflag=fullblock conv=fsync 1>>$PROD_LOG 2>&1
			let STATUS=STATUS+$?
			sync

			if [ $STATUS -eq 0 ]; then
				printStatus "resizing partition ..."
				e2fsck -f $EMMC_PART 1>>$PROD_LOG 2>&1
				resize2fs -p $EMMC_PART 1>>$PROD_LOG 2>&1
				let STATUS=STATUS+$?
				if [ $STATUS -ne 0 ]; then
					printStatus "rootfs on $EMMC_PART could not be extended to it's maximum size!" 1
				fi
			fi
		fi
	else
		printStatus "No wic-Image $IMAGE_WIC found in $FW_DIR" 1
		STATUS=1
	fi
	flashResult eMMC-ROOTFS $STATUS
	clean_fw_dir
	return $STATUS
}


##############################################################################
#	** Initial Partitioning ** 
#	prepare eMMC Storage with Partitions needed for using A/B 
#	Update approach
# 	
	#parameter: please provide path to a valid SWUpdate archive
#	* the .swu archive should provide rootfs.tar.gz and fitImage a file 
#	* build the image by using bitbake swupate-img
#	
##############################################################################
flash_emmc_swu(){
	local status=0
	local cur_par=$(findmnt -n -o SOURCE /)
	local total_size=0

	getProductionFiles $SWU_ARCHIVE
	
	if [ ! -f $FW_DIR/$SWU_ARCHIVE ]; then
		printStatus "$SWU_ARCHIVE not available in $FW_DIR" 1
		return 1
	fi

	umount_dev_mmc_partitions `basename $EMMC_DEV`

	# calculate size of partitions on eMMC
	# subtract 2*rootfs_size and bootfs_size to get the available free space 
	# for the userfs Partition 

	total_size=$(sfdisk -l ${EMMC_DEV} | grep "Disk /dev/"  | cut -d ' ' -f5)
	total_size_mb=$(expr $total_size / 1024 / 1024 )

	echo "totalsize: $total_size"
	echo "totalsize: $total_size_mb M"

	userfs_size=$(expr $total_size_mb - $(($rootfs_size * 2)) - $bootfs_size)
	
	printStatus "calculated total_size: $total_size_mb MiB"
	printStatus "calculated userfs_size: $userfs_size MiB"

	dd if=/dev/zero of=${EMMC_DEV} bs=512 count=1 1>/dev/null 2>&1
	printStatus "creating partition table"
	sfdisk ${EMMC_DEV} 1>>$PROD_LOG 2>&1  << EOF
label:dos
size=${bootfs_size}M,
size=${rootfs_size}M,
size=${rootfs_size}M,
size=${userfs_size}M,
EOF

	for i in 1 2 3 4 ; do
		format_to_ext4 ${EMMC_DEV}p${i}
		let status=status+$?
	done

	if [ $status -ne 0 ]; then
		printStatus "failed to format ext4 partitions" 1 
		return $status
	fi

	if [ ! -x $swupdate_bin ];then 
		printStatus "$swupdate_bin ist not installed or executable." 1
		return 1
	fi
		
	# only possible to install root_A if we do initial partitioning/setup!!!
	printStatus "installing $SWU_ARCHIVE. Select $update_A"
	$swupdate_bin -f $UPDATE_CFG -e $update_A -i $FW_DIR/$SWU_ARCHIVE
	let status=status+$?
	#disable bootcounter in u-boot - no fallback installation in production setup. 
	fw_setenv upgrade_available 0
	fw_setenv bootcount 0
	flashResult EMMC-SWU $status
	return $status
}

set_time() {
	STATUS=0
	if [ $# -ne 2 ]; then
		printStatus "two parameter needed (date time): 2018-02-01 12:34:32" 1
		return 1
	fi

	date=$(date -s "$1 $2")
	STATUS=$?
	hwclock --systohc 1>>$PROD_LOG 2>&1
	let STATUS=STATUS+$?
	printStatus "setting system time RTC to $date" $STATUS
	return $STATUS
}

########################################################################
ntp_sync() {
	STATUS=0
	printStatus "synchonizing system time from timeserver..."
	ntpdate $NTP_SERVER 1>>$PROD_LOG 2>&1
	STATUS=$?
	printStatus "synchonizing system time from timeserver result:" $STATUS
	if [ $STATUS -ne 0 ]; then
		return $STATUS
	fi
	hwclock --systohc 1>>$PROD_LOG 2>&1
	return 0
}
########################################################################
get_lan1_mac_otp() {
	mac0=$(getOtpRegister $MAC0_OCOTP_REG | cut -c 3-)
	mac1=$(getOtpRegister $MAC1_OCOTP_REG | cut -c 7-)
	echo $mac1$mac0
	return 0
}

########################################################################
get_lan2_mac_otp() {
	mac2=$(getOtpRegister $MAC2_OCOTP_REG | cut -c 3-)
	mac1=$(getOtpRegister $MAC1_OCOTP_REG | cut -c 3-6)
	echo $mac2$mac1
	return 0
}

########################################################################
# setting mac addresses for eth0 temporary in ram
set_lan1_mac_temp() {
	l=${#1}

	if [ ! $l -eq 12 ]; then
		printStatus "Invalid MAC length!" 1
		return 1
	fi

	mac=$1
	ifconfig eth0 down 1>>$PROD_LOG 2>&1
	ifconfig eth0 hw ether ${mac:0:2}:${mac:2:2}:${mac:4:2}:${mac:6:2}:${mac:8:2}:${mac:10:2} 1>>$PROD_LOG 2>&1
	printStatus "setting MAC temporarily to $mac" $?
}

########################################################################
# setting mac addresses for eth0 and eth1 temporary in ram
set_lan_mac_temp() {
	l=${#1}

	if [ ! $l -eq 12 ]; then
		printStatus "Invalid MAC length!" 1
		return 1
	fi

	mac=$1
	mac1=$(awk "BEGIN {print 0x$mac+1; exit}")
	mac1=$(printf '%x' $mac1)
	ifconfig eth0 down 1>>$PROD_LOG 2>&1
	ifconfig eth0 hw ether ${mac:0:2}:${mac:2:2}:${mac:4:2}:${mac:6:2}:${mac:8:2}:${mac:10:2} 1>>$PROD_LOG 2>&1
	ifconfig eth1 down 1>>$PROD_LOG 2>&1
	ifconfig eth1 hw ether ${mac1:0:2}:${mac1:2:2}:${mac1:4:2}:${mac1:6:2}:${mac1:8:2}:${mac1:10:2} 1>>$PROD_LOG 2>&1
	printStatus "setting MACs temporarily to $mac and $mac1" $?
}

########################################################################
# setting otp for mac for solo/dual/quad boards
# the mac is read from parameter $1
set_lan1_mac_otp() {
	if [ ! ${#1} -eq 12 ]; then
		printStatus "Invalid MAC length!" 1
		return 1
	fi

	mac=$1
	mac0=$(getOtpRegister $MAC0_OCOTP_REG)
	mac1=$(getOtpRegister $MAC1_OCOTP_REG)
	current_mac=$(get_lan1_mac_otp)

	if [ "$current_mac" == "000000000000" ]; then
		mac_otp0=$(echo $1 | cut -c 5-)
		mac_otp1=$(echo $1 | cut -c -4)
		let ret=0
		setOtpRegister $MAC0_OCOTP_REG "0x$mac_otp0"
		let ret=ret+$?
		setOtpRegister $MAC1_OCOTP_REG "0x$mac_otp1"
		let ret=ret+$?
		if [ $ret -ne 0 ]; then
			printStatus "burning MAC $1 to OTPs" 1
		fi
	else
		mac=$(echo "$1" | tr '[:upper:]' '[:lower:]')
		current_mac=$(echo "$current_mac" | tr '[:upper:]' '[:lower:]')
		if [ "$current_mac" = "$mac" ]; then
			printStatus "MAC OTPs already set to $mac!" 0
			return 0
		else
			printStatus "MAC OTPs already set (mismatch MAC: $current_mac != $mac)!" 1
			return 1
		fi
	fi
	return 0
}

########################################################################
# setting otp for mac 1 and mac 2 for mx6ul/mx6ull/mx8mm boards
# the first mac is read from parameter $1
# the second mac is set to mac1 + 1
set_lan_mac_otp() {
	if [ ! ${#1} -eq 12 ]; then
		printStatus "Invalid MAC length!" 1
		return 1
	fi

	mac=$1
	mac_otp0=$(getOtpRegister $MAC0_OCOTP_REG)
	mac_otp1=$(getOtpRegister $MAC1_OCOTP_REG)
	mac_otp2=$(getOtpRegister $MAC2_OCOTP_REG)
	current_mac=$(get_lan1_mac_otp)
	current_mac1=$(get_lan2_mac_otp)

	if [ "$current_mac" == "000000000000" ] && [ "$current_mac1" == "000000000000" ]; then
		mac1=$(awk "BEGIN {print 0x$mac+1; exit}")
		mac1=$(printf '%x' $mac1)
		mac_otp0=$(echo $mac | cut -c 5-)
		mac_otp1=$(echo $mac | cut -c -4)
		mac_otp1=$(echo $mac1 | cut -c 9-)"$mac_otp1"
		mac_otp2=$(echo $mac1 | cut -c -8)
		let ret=0
		setOtpRegister $MAC0_OCOTP_REG "0x$mac_otp0"
		let ret=ret+$?
		setOtpRegister $MAC1_OCOTP_REG "0x$mac_otp1"
		let ret=ret+$?
		setOtpRegister $MAC2_OCOTP_REG "0x$mac_otp2"
		let ret=ret+$?
		if [ $ret -ne 0 ]; then
			printStatus "burning MAC $1 to OTPs" 1
		else
			setOtpRegister $LOCK_OCOTP_REG $MAC_LOCK
			printStatus "MAC $1 burned to OTPs succesfully" 0
		fi
	else
		mac=$(echo "$mac" | tr '[:upper:]' '[:lower:]')
		mac1=$(awk "BEGIN {print 0x$mac+1; exit}")
		mac1=$(printf '%.12x' $mac1)
		current_mac=$(echo "$current_mac" | tr '[:upper:]' '[:lower:]')
		current_mac1=$(echo "$current_mac1" | tr '[:upper:]' '[:lower:]')
		if [ "$current_mac1" != "$mac1" ]; then
			printStatus "MAC OTPs already set (mismatch MAC2: $current_mac1 != $mac1)!" 1
			return 1
		fi
		if [ "$current_mac" != "$mac" ]; then
			printStatus "MAC OTPs already set (mismatch MAC1: $current_mac != $mac)!" 1
			return 1
		fi
		printStatus "MAC OTPs already set to MAC1: $mac, MAC2: $mac1" 0
		return 0
	fi
}

########################################################################
# setting otp for mac 2 for mx6ul/mx6ull/mx8mm boards
# the second mac is read from parameter $1
set_lan2_mac_otp() {
	if [ ! ${#1} -eq 12 ]; then
		printStatus "Invalid MAC length!" 1
		return 1
	fi

	mac=$1
	mac_otp1=$(getOtpRegister $MAC1_OCOTP_REG)
	mac_otp2=$(getOtpRegister $MAC2_OCOTP_REG)
	current_mac=$(get_lan1_mac_otp)
	current_mac1=$(get_lan2_mac_otp)

	if [ "$current_mac1" == "000000000000" ]; then
		mac_otp1=$(echo $mac | cut -c 9-)$(echo $mac_otp1 | cut -c 3-6)
		mac_otp2=$(echo $mac | cut -c -8)
		let ret=0
		setOtpRegister $MAC1_OCOTP_REG "0x$mac_otp1"
		let ret=ret+$?
		setOtpRegister $MAC2_OCOTP_REG "0x$mac_otp2"
		let ret=ret+$?
		if [ $ret -ne 0 ]; then
			printStatus "burning MAC $1 to OTPs" 1
		else
			setOtpRegister $LOCK_OCOTP_REG $MAC_LOCK
			printStatus "MAC $1 burned to OTPs succesfully" 0
		fi
	else
		mac=$(echo "$mac" | tr '[:upper:]' '[:lower:]')
		current_mac=$(echo "$current_mac" | tr '[:upper:]' '[:lower:]')
		current_mac1=$(echo "$current_mac1" | tr '[:upper:]' '[:lower:]')
		if [ "$current_mac1" != "$mac" ]; then
			printStatus "MAC OTPs already set (mismatch MAC2: $current_mac1 != $mac)!" 1
			return 1
		fi
		printStatus "MAC OTPs already set to MAC1: $current_mac, MAC2: $mac" 0
		return 0
	fi
}

########################################################################
# setting otp for mac for solo/dual/quad boards
# the mac is read from input
set_lan1_mac_otp_user() {
	current_mac=$(get_lan1_mac_otp)

	if [ "$current_mac" == "000000000000" ]; then
		echo "Set i.MX Ethernet Adapter MAC to OTP (irreversible!)"
		ret="n"
		until [ "$ret" = "y" ]; do
			echo -n "Enter lan1 MAC address [XXXXXXXXXXXX]: "
			read mac
			echo -n "lan1 MAC address $mac correct [y/n] ?: "
			read ret
		done
		set_lan1_mac_otp $mac
	else
		printStatus "MAC OTPs already set!"
	fi
}

########################################################################
# setting otp for mac1 and mac2 for mx6ul/mx6ull/mx8mm boards
# the first mac is read from input
# the second mac is set to mac1 + 1
set_lan_mac_otp_user() {
	current_mac=$(get_lan1_mac_otp)
	current_mac1=$(get_lan2_mac_otp)

	if [ "$current_mac" == "000000000000" ] && [ "$current_mac1" == "000000000000" ]; then
		echo "Set i.MX Ethernet Adapter MAC to OTP (irreversible!)"
		ret="n"
		until [ "$ret" = "y" ]; do
			echo -n "Enter LAN1 mac address [XXXXXXXXXXXX]: "
			echo -n "(LAN2 MAC will be set to LAN1 MAC + 1): "
			read mac
			echo -n "LAN1 MAC address $mac correct [y/n] ?: "
			read ret
		done
		set_lan_mac_otp $mac
	else
		printStatus "MAC OTPs already set!"
	fi
}

########################################################################
# setting otp for mac2 for mx6ul/mx6ull/mx8mm boards
# the second mac is read from input
set_lan2_mac_otp_user() {
	current_mac1=$(get_lan2_mac_otp)

	if [ "$current_mac1" == "000000000000" ]; then
		echo "Set i.MX Ethernet Adapter MAC to OTP (irreversible!)"
		ret="n"
		until [ "$ret" = "y" ]; do
			echo -n "Enter LAN2 mac address [XXXXXXXXXXXX]: "
			read mac
			echo -n "LAN2 MAC address $mac correct [y/n] ?: "
			read ret
		done
		set_lan2_mac_otp $mac
	else
		printStatus "MAC OTPs already set!"
	fi
}

########################################################################
# Parameter:
#1. Verzeichnis fuer MAC-Adressen, z.b. USB-Stick)
#2. Offset fuer Mac Adressen z.B.: 70:82:0e:99:
writeMac_auto() {
	#start mac-adress (16-Byte range)
	#macFormattedOffset=70:82:0e:99:
	num=0
	numberDevices=256
	MAC_DIR=$1
	macFormattedOffset=$2

	if [ "$#" != 2 ]; then
		echo "wrong parameter count, need 2 parameters:- MAC address dir and MAC offset dir"
		echo "writeMac_auto 'directory' '70:82:0e:99:'"
		return 1
	fi

	check_usedMac $MAC_DIR
	if [ $? == 0 ]; then
		#read variable part of MAC-adress as HEX from file
		#and extract low and  high byte
		if [ ! -s $MAC_DIR/nextMac.txt ]; then
			return 1
		fi
		read mac <$MAC_DIR/nextMac.txt
		highMac=$(echo $mac | cut -c 3-4)
		lowMac=$(echo $mac | cut -c 5-6)

		# formattedMac needed for fw_env tool
		formattedMac=$(echo $macFormattedOffset$highMac:$lowMac)
		echo "setting eth0 MAC adress for device:" $formattedMac
		echo $formattedMac >>$MAC_DIR/usedMac.txt
		fw_setenv ethaddr $formattedMac

		#increment variable part of MAC and write it to file
		#for next board in production
		let mac=$mac+1
		echo "writing next calculated MAC to file $MAC_DIR/nextMac.txt"
		echo $(printf "0x%04x" $mac) >$MAC_DIR/nextMac.txt
	fi
}

# Parameter:
#1. Verzeichnis fuer MAC-Adressen, z.b. USB-Stick)
#2. Offset fuer Mac Adressen z.B.: 70:82:0e:99:

#Rueckgabewerte:
#0: MAC bereits vergeben
#1: noch keine MAC vergeben

########################################################################
check_usedMac() {
	# check if hardware already passed the production and has a valid MAC
	if [ "$#" != 1 ]; then
		echo "wrong parameter count, need 1 parameter:- MAC address dir "
		echo "check_usedMac 'directory'"
		return 255
	fi
	MAC_DIR=$1
	fw_printenv ethaddr >/dev/null
	if [ $? == 0 ]; then
		mac=$(fw_printenv ethaddr | cut -c 9-)
		grep -q $mac $MAC_DIR/usedMac.txt
		if [ $? == 0 ]; then
			#echo "Steuerung bereits getestet!" | tee -a $PROD_LOG
			return 1
		fi
	fi
	return 0
}

########################################################################
#Parameter 1: Bootkonfiguration
burn_bootconfiguration() {
	if [ "$#" != 1 ] && [ "$#" != 2 ]; then
		echo "burn_bootconfiguration needs 1 or 2 parameters"
		return 1
	fi

	# ueberpruefe: OTP fuer Bootkonfiguration noch unberührt
	if [ "0x00000000" = $(getOtpRegister $BOOT_CONFIG_OCOTP_REG) ] ||
		[ "0x08000000" = $(getOtpRegister $BOOT_CONFIG_OCOTP_REG) ]; then
		setOtpRegister $BOOT_CONFIG_OCOTP_REG $1
		if [ $? -ne 0 ]; then
			printStatus "bootconfiguration <$1> not fused to OTPs" 1
		elif [ "$2" != "" ]; then
			setOtpRegister $BOOT_CONFIG2_OCOTP_REG $2
		fi
		if [ $? -ne 0 ]; then
			printStatus "setting bootconfiguration <$1> <$2>: failed" 1
		else
			printStatus "bootconfiguration <$1> <$2> fused to OTPs>" 0
		fi
	else
		printStatus "bootconfiguration already set"
	fi

	# Wert ueberpruefen
	echo "$(getOtpRegister $BOOT_CONFIG_OCOTP_REG)" | grep -q $1
	if [ $? != 0 ]; then
		printStatus "incorrect bootconfiguration <$(getOtpRegister $BOOT_CONFIG_OCOTP_REG)> instead of <$1>" 1
		return 1
	else
		printStatus "correct bootconfiguration <$(getOtpRegister $BOOT_CONFIG_OCOTP_REG)> = <$1>" 0
	fi
	return 0
}

########################################################################
burn_serial_no() { #parameter 1: 9-digit serial no
	if [ "$#" != 2 ]; then
		printStatus "$# parameters set: <$@>"
		printStatus "need 2 parameters: serial no and type (0: SOM 1: board)" 1
		printStatus "example: burn_serial_no 030002470 0"
		printStatus "burn SOM serial no 030002470"
		return 1
	fi
	bType=$2
	if [ $bType == 0 ]; then
		GP_OTP_REG=$GP1_OCOTP_REG
		GP_LOCK=$GP1_LOCK
	else
		if [ $bType == 1 ]; then
			GP_OTP_REG=$GP2_OCOTP_REG
			GP_LOCK=$GP2_LOCK
		else
			printStatus "need serial no type 0 (SOM) or 1 (board)"
			return 1
		fi
	fi
	nSerial=$1
	# check size
	nSize=${#1}
	if [ $nSize -ne 9 ]; then
		#check for leading sign
		if [ $nSize -ne 10 ]; then
			printStatus "serial no not fused, wrong size: $nSize, expected: 9 digits" 1
			return 1
		else
			# checking leading character
			cLeadingChar=$(echo $nSerial | cut -c1)
			cLeadingChar=$(printf '%d' \'$cLeadingChar)
			if [ "$cLeadingChar" != "35" -a "$cLeadingChar" != "167" ]; then

				printf 'compare with: %d\n' \'"§"
				printStatus "serial no not fused, wrong leading char: <$cLeadingChar>" 1
				return 1
			else
				# eliminate leading character
				nSerial=$(echo $nSerial | cut -c2-)
			fi
		fi
	fi
	# check numer
	case $nSerial in
	*[!0-9]* | '')
		printStatus "Error: not a number" 1
		return 1
		;;
	*)
		printStatus "a number"
		;;
	esac

	echo "nSerial=<$nSerial>"
	nSerial_decimal=$nSerial
	# eliminate leading zeros (otherwise printf assumes octal number)
	nSerial=$(expr $nSerial + 0)
	nSerial=$(printf '0x%x\n' $nSerial)
	echo "nSerial=<$nSerial>"
	# check: serial number not set yet
	if [ "0x00000000" = $(getOtpRegister $GP_OTP_REG) ]; then
		setOtpRegister $GP_OTP_REG $nSerial
		if [ $? -ne 0 ]; then
			printStatus "burning serial number $nSerial to OTPs" 1
		else
			printStatus "serial no <$nSerial> fused to register no <$GP_OTP_REG>" 0
			# lock gp fuses
			setOtpRegister $LOCK_OCOTP_REG $GP_LOCK
		fi
	else
		# Serialnumber is already set (check if it matches)
		if [ "$nSerial_decimal" == "$(printf '%.9d\n' $(getOtpRegister $GP_OTP_REG))" ]; then
			printStatus "serial no already set" 0
			return 0
		else
			printStatus "serial number already set (mismatch) $nSerial_decimal : $(get_serial_no $bType)" 1
			return 1
		fi
	fi
	return 0
}

########################################################################
get_serial_no() {
	if [ "$#" != 1 ]; then
		printStatus "need 1 parameter: type (0: SOM 1: board)" 1
		printStatus "example: get_serial_no 0"
		printStatus "get SOM serial no"
		return 1
	fi
	if [ $1 == 0 ]; then
		# SOM serial number
		#		printStatus "bType=0 (SOM)" 0
		GP_OTP_REG=$GP1_OCOTP_REG
	else
		if [ $1 == 1 ]; then
			#			printStatus "bType=1 (board)" 0
			GP_OTP_REG=$GP2_OCOTP_REG
		else
			printStatus "need serial no type 0 (SOM) or 1 (board)" 1
			return 1
		fi
	fi
	nSerial=$(printf '%.9d\n' $(getOtpRegister $GP_OTP_REG))
	if [ $1 == 0 ]; then
		echo "SOM SerialNo: $nSerial"
	else
		echo "Board SerialNo: $nSerial"
	fi
	return 0
}

########################################################################
get_fw_version() {
	issue=$(cat /etc/issue)
	issue=${issue%" \n \l"}
	echo "Firmware Version: $issue"
	return 0
}
